#pragma once
#define TIXML_USE_STL
#include "tinyxml.h"
#include "tipconst.h"
#include "AgentDLL.h"
#include "DataContext.h"
#include "Binding.h"
#include "XMLDataProvider.h"
#include "StatManager.h"
#include "ScriptletUtil.h"
#include "checkinexecution.h"

#include <vector>
#include <algorithm>
#include <atlrx.h>

namespace tipcommon
{	
	/**
	* @enum eInstallInfo
	* ġ   (̼ġ, , ֽŹ)
	*/
	struct InstallInfo
	{
		enum type {
			NOTINSTALLED,
			OLDVERSION,
			RECENTVERSION,
		};
	};

	/**
	* @enum eInstallStatus
	* ġ  
	*/
	struct InstallStatus
	{
		enum type
		{
			INIT		= 0x01,		// ġ غ
			DOWNLOAD	= 0x02,		//  ٿε 
			READY		= 0x04,		// ġ 
			INSTALL		= 0x08,		// ġ 
			UPDATE		= 0x10,		// Ʈ 
			FINISH		= 0x20,		// ġ Ϸ
			CANCEL		= 0x40,		// ġ 
			FAIL		= 0x80,		// ġ 
			FINISH_SET  = FINISH | CANCEL | FAIL //  
		};
	};
	class TipServiceData;
	class TipXMLValidater;
	/**
	* @class TipDocument
	* ťƮ Ŭ (    )
	*/
	class TipDocument
	{
	public:
		TipDocument(void);
		virtual ~TipDocument(void);

		virtual BOOL Open();
		virtual BOOL Close(); 

		void SetTempXmlDoc(TiXmlDocument* xmlDoc)
		{
			m_DocTemp = xmlDoc;
		}
		void AddDataContext(LPCTSTR name);
		IDataContext* GetChild(LPCTSTR name);
		IDataContext* GetRoot();

		CComVariant GetProperty(LPCTSTR key);
		void SetProperty(LPCTSTR key, CComVariant data);

		int GetCountOfServiceData(LPCTSTR name);
		TipServiceData* GetServiceData(LPCTSTR name, int index);
		void ClearServiceData();
		void AddServiceData(TipServiceData* service);

		void SetDefaultListIndex(int index);
		int GetDefaultListIndex();

		/*
		* @fn		ServiceModel* GetServiceModel(int index)
		* @brief	ش ε   ͸ ϴ Լ
		*/
		TipServiceData* GetService(int index) { return m_ServiceList.at(index); }
		int GetServiceCount() { return m_ServiceList.size(); }
		TipServiceData* SearchService(const TCHAR* guid);
		void OpenUrlWithDefaultBrowser(LPCTSTR url);

	//
	// BTS-102 101022 Ȳ߰
	//  ۽Ų α׷ GUID
	//
	public:
		CString m_StartupProgramGuid;

	protected:
		TiXmlDocument* m_DocTemp;
		IDataContext* m_rootDataContext;
		IStatManager* m_StatManager;
		std::vector<TipServiceData*> m_ServiceList;
		TipXMLValidater* m_valueValidater;


	private:
		BOOL LoadXmlDoc();

	};

	class TipServiceData
	{
	protected:
		AgentDLLWrapper* m_agentDllWrapper;
		IDataContext* m_datacontext;
		TipDocument* m_document;

	public:
		TipServiceData(TipDocument* document, IDataContext* datacontext);
		virtual ~TipServiceData();

		void RegisterAgentDll();
		void UnregisterAgentDll();
		AgentDLLWrapper* GetAgentDll();

		virtual void SetInstallInfo(InstallInfo::type info);
		/** 
		* @fn		int GetInstallInfo()
		* @brief	 ġ  ϴ Լ
		*/
		InstallInfo::type GetInstallInfo() 
		{
			ATLASSERT(m_datacontext);
			CComVariant var = m_datacontext->GetProperty(_T("installinfo"));
			return static_cast<InstallInfo::type>(var.intVal);
		}
		/** 
		* @fn		void SetInstallStatus(int status)
		* @brief	 ġ/Ʈ ¸ ϴ Լ
		*/
		void SetInstallStatus(InstallStatus::type status)
		{
			ATLASSERT(m_datacontext);
			m_datacontext->SetProperty(_T("installstatus"), CComVariant(status));
		}
		/** 
		* @fn		int GetInstallStatus()
		* @brief	 ġ/Ʈ ¸ ϴ Լ
		*/
		InstallStatus::type GetInstallStatus()
		{ 
			ATLASSERT(m_datacontext);
			CComVariant var = m_datacontext->GetProperty(_T("installstatus"));
			return static_cast<InstallStatus::type>(var.intVal);
		}

		void SetInstallProgressPos(double pos)
		{
			ATLASSERT(m_datacontext);
			m_datacontext->SetProperty(_T("progress_position"), pos);
		}
		double GetInstallProgressPos() 
		{ 
			ATLASSERT(m_datacontext);
			CComVariant var = m_datacontext->GetProperty(_T("progress_position"));
			return var.dblVal;
		}
		/** 
		* @fn		Tvoid SetInstallSelect(bool select)
		* @brief	񽺸  ϴ Լ
		*/
		void SetInstallSelect(BOOL select)
		{
			ATLASSERT(m_datacontext);
			m_datacontext->SetProperty(_T("selected_service"), CComVariant(select));
		}
		/** 
		* @fn		bool GetInstallSelect()
		* @brief	񽺰 õǾ ϴ Լ
		*/
		BOOL GetInstallSelect()
		{
			ATLASSERT(m_datacontext);
			CComVariant var = m_datacontext->GetProperty(_T("selected_service"));
			return ScriptUtil::Varient2BOOL(&var);;
		}

		//operation
		//helper
		CString GetID() 
		{
			ATLASSERT(m_datacontext);
			CComVariant var = m_datacontext->GetProperty(_T("guid"));
			return CString(var.bstrVal);
		}
		CString GetName() 
		{
			ATLASSERT(m_datacontext);
			CComVariant var = m_datacontext->GetProperty(_T("name"));
			return CString(var.bstrVal);
		}
		int GetMinOS()
		{
			ATLASSERT(m_datacontext);
			int nVersion = tipbase::WindowVersion::win_2000;
			CComVariant var = m_datacontext->GetProperty(_T("min_os"));
			if (var.vt == VT_BSTR)
			{
				nVersion = _wtoi(var.bstrVal);
			}
			else if(var.vt != VT_EMPTY)
			{
				nVersion = var.lVal;
			}
			return nVersion;
		}
		CString GetAgentDataURL() 
		{
			ATLASSERT(m_datacontext);
			CComVariant var = m_datacontext->GetProperty(_T("(agentdllxml.url)"));
			return CString(var.bstrVal);
		}
		CString GetIntroduceURL() 
		{
			ATLASSERT(m_datacontext);
			CComVariant var = m_datacontext->GetProperty(_T("(introduce.url)"));
			return CString(var.bstrVal);
		}
		CString GetSetupURL() 
		{
			ATLASSERT(m_datacontext);
			CComVariant var = m_datacontext->GetProperty(_T("(setupfile.url)"));
			return CString(var.bstrVal);
		}
		CString GetUpdateInfoURL() 
		{
			ATLASSERT(m_datacontext);
			CComVariant var = m_datacontext->GetProperty(_T("(update.infourl)"));
			return CString(var.bstrVal);
		}
		CString GetIconURL() 
		{
			ATLASSERT(m_datacontext);
			CComVariant var = m_datacontext->GetProperty(_T("(icon.url)"));
			return CString(var.bstrVal);
		}
		CString GetFolderName() 
		{
			ATLASSERT(m_datacontext);
			CComVariant var = m_datacontext->GetProperty(_T("foldername"));
			if (var.vt == VT_EMPTY )
			{
				CString str;
				str.Format(L"TempDir%u", GetTickCount());
				return CString(str);
			}
			return CString(var.bstrVal);
		}
		CString GetUpdateCopy() 
		{
			ATLASSERT(m_datacontext);
			CComVariant var = m_datacontext->GetProperty(_T("(update.copy)"));
			return CString(var.bstrVal);
		}
		BOOL IsShowPath()
		{
			ATLASSERT(m_datacontext);
			CComVariant var = m_datacontext->GetProperty(_T("showpath"));
			return ScriptUtil::Varient2BOOL(&var);
		}
		BOOL IsUpdateShowInfo()
		{
			ATLASSERT(m_datacontext);
			CComVariant var = m_datacontext->GetProperty(_T("(update.showinfo)"));
			return ScriptUtil::Varient2BOOL(&var);
		}
		BOOL IsUpdateSilent()
		{
			ATLASSERT(m_datacontext);
			CComVariant var = m_datacontext->GetProperty(_T("(update.silent)"));
			return ScriptUtil::Varient2BOOL(&var);
		}
		BOOL IsAgreementShow()
		{
			ATLASSERT(m_datacontext);
			CComVariant var = m_datacontext->GetProperty(_T("(agreement.show)"));
			return ScriptUtil::Varient2BOOL(&var);
		}
		CString GetAgreementURL() 
		{
			ATLASSERT(m_datacontext);
			CComVariant var = m_datacontext->GetProperty(_T("(agreement.url)"));
			return CString(var.bstrVal);
		}
		//helper
		LPCTSTR GetStartViewURL() 
		{ 
			ATLASSERT(m_datacontext);
			CComVariant var = m_datacontext->GetProperty(_T("(startview.url)"));
			return var.bstrVal;
		}
		float GetCapacity() 
		{
			ATLASSERT(m_datacontext);
			float capacity = 0.f;
			CComVariant var = m_datacontext->GetProperty(_T("capacity"));
			if (var.vt == VT_EMPTY)
			{
				ATLTRACE(L"********************* Warinig ************************* capacity is empty\n");
#ifdef _DEBUG
				capacity = 5.5;
#endif
				return capacity;
			}
			else if (var.vt == VT_BSTR)
			{
				capacity = _wtof(var.bstrVal);
			}
			else
			{
				capacity = var.fltVal;
			}
			return capacity;
		}
		BOOL IsDefaultInstall()
		{
			ATLASSERT(m_datacontext);
			CComVariant var = m_datacontext->GetProperty(_T("defaultinstall"));
			return ScriptUtil::Varient2BOOL(&var);
		}
		void SetVisible(BOOL visible)
		{
			ATLASSERT(m_datacontext);
			m_datacontext->SetProperty(_T("visible"), CComVariant(visible));
		}
		BOOL IsChangedInstallInfo(InstallInfo::type& newInfo)
		{
			AgentDLLWrapper* agentDllWrapper = NULL;
			agentDllWrapper = GetAgentDll();
			if (agentDllWrapper == NULL)
				return FALSE;
			NAVERAPP_INSTALL_STATUS InstallStatus;
			agentDllWrapper->GetInstalledStatus(InstallStatus);

			switch(InstallStatus)
			{
			case NotInstalled:
				newInfo = InstallInfo::NOTINSTALLED;
				break;
			case OldVersion:
				newInfo = InstallInfo::OLDVERSION;
				break;
			case LatestVersion:
				newInfo = InstallInfo::RECENTVERSION;
				break;
			}
			BOOL bChanged =  (newInfo != GetInstallInfo());

			return bChanged;
		}
		void SetInstallErrorText(LPCTSTR errText)
		{
			ATLASSERT(m_datacontext);
			m_datacontext->SetProperty(_T("installerrortext"), CComVariant(errText));
		}
		void SetInstallErrorID(int id)
		{
			ATLASSERT(m_datacontext);
			m_datacontext->SetProperty(_T("installerrorid"), CComVariant(id));
		}
		
		CString GetProcessor()
		{
			ATLASSERT(m_datacontext);
			CComVariant var = m_datacontext->GetProperty(_T("processor"));
			if ( var.vt != VT_BSTR )
			{
				CString str;
				if ( var.vt == VT_I4)
				{					
					str.Format(_T("%dbit"), var.intVal);
				}
				return str;
			}
			return CString(var.bstrVal);
		}

		BOOL IsPartner();

		CString GetBigIntroImageUrl();		

		BOOL IsNewIcon();

		// ֿɺ URL
		CString GetMainfuncURL();

		IDataContext* GetDataContext() { return m_datacontext; }
	};

	class TipXMLValidater: public IXMLValueValidater
	{
	public:
		TipXMLValidater():m_nInvalidNumber(0) {}
		enum valueType
		{
			UrlType = 0,
			GuidType,
			NumberType,
			BooleanType,
			StringType,
		};
			

		// override
		BOOL IsValid(LPCTSTR key, LPCTSTR data) = 0;
		BOOL IsXMLDataValid() 
		{ 
			return (m_nInvalidNumber==0); 
		}
		void Fix(LPCTSTR key, LPCTSTR data, LPTSTR newData, int len);

	protected:
		BOOL IsValidURL(LPCTSTR data);
		BOOL IsValidGUID(LPCTSTR data);
		BOOL IsValidNumber(LPCTSTR data);
		BOOL IsValidBoolean(LPCTSTR data);
		BOOL IsValidString(LPCTSTR data) { return TRUE; }

		BOOL IsValidWithRegExp(LPCTSTR data, LPCTSTR regexp);
		int m_nInvalidNumber;
	};

	BOOL IsWow64();
};
